#  SPDX-License-Identifier: Apache-2.0
"""
Python Package for controlling Tesla API.

For more details about this api, please refer to the documentation at
https://github.com/zabuldon/teslajsonpy
"""

from teslajsonpy.exceptions import HomelinkError
from teslajsonpy.homeassistant.vehicle import VehicleDevice


class TriggerHomelink(VehicleDevice):
    """Home-Assistant class for trigger homelink of Tesla vehicles."""

    def __init__(self, data, controller):
        """Initialize the trigger homelink for the vehicle.

        Parameters
        ----------
        data : dict
            The trigger homelink for a Tesla vehicle.
            https://tesla-api.timdorr.com/vehicle/commands/homelink
        controller : teslajsonpy.Controller
            The controller that controls updates to the Tesla API.

        Returns
        -------
        None

        """
        super().__init__(data, controller)
        self.type = "trigger homelink"
        self.hass_type = "button"
        self.name = self._name()
        self.uniq_name = self._uniq_name()
        self.enabled_by_default = False

        self._longitude = None
        self._latitude = None
        self._homelink_device_count = None
        self._homelink_nearby = None
        self._homelink_available = False

    @staticmethod
    def has_battery() -> bool:
        """Return whether the device has a battery."""
        return False

    def available(self) -> bool:
        """Return whether homelink is available."""
        return self._homelink_available

    async def async_update(self, wake_if_asleep=False, force=False):
        """Update the trigger homelink of the vehicle."""
        await super().async_update(wake_if_asleep=wake_if_asleep, force=force)
        self.refresh()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        data = self._controller.get_drive_params(self._id)
        if data:
            if data.get("native_location_supported"):
                self._longitude = data.get("native_longitude")
                self._latitude = data.get("native_latitude")
            else:
                self._longitude = data.get("longitude")
                self._latitude = data.get("latitude")
        data = self._controller.get_state_params(self._id)
        if data:
            self._homelink_device_count = data.get("homelink_device_count")
            self._homelink_nearby = data.get("homelink_nearby")
        self._homelink_available = bool(self._homelink_device_count)

    async def trigger_homelink(self) -> None:
        """Trigger Homelink."""
        await self.async_update(wake_if_asleep=True, force=True)
        if self._latitude is not None and self._longitude is not None:
            if not self._homelink_device_count:
                raise HomelinkError(f"No homelink devices added to {self.car_name()}.")
            if not self._homelink_nearby:
                raise HomelinkError(f"No homelink devices near {self.car_name()}.")
            data = await self._controller.api(
                "TRIGGER_HOMELINK",
                path_vars={"vehicle_id": self._id},
                lat=self._latitude,
                lon=self._longitude,
                wake_if_asleep=True,
            )
            if data and data.get("response"):
                result = data["response"].get("result")
                reason = data["response"].get("reason")
                if result is False:
                    raise HomelinkError(f"Error calling trigger_homelink: {reason}")



#  SPDX-License-Identifier: Apache-2.0
"""
Python Package for controlling Tesla API.

For more details about this api, please refer to the documentation at
https://github.com/zabuldon/teslajsonpy
"""
import logging

from typing import Optional, Text, List
from teslajsonpy.homeassistant.vehicle import VehicleDevice

_LOGGER = logging.getLogger(__name__)


class VehicleDataSensor(VehicleDevice):
    """Home-Assistant vehicle data class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the vehicle data sensor.

        Parameters
        ----------
        data : dict
            The base state for a Tesla vehicle.
            https://tesla-api.timdorr.com/vehicle/state/data
        controller : teslajsonpy.Controller
            The controller that controls updates to the Tesla API.

        Returns
        -------
        None

        """
        super().__init__(data, controller)
        self.__state: Optional[str] = None

        self.type: Text = "sensor"
        self.hass_type: Text = "sensor"
        # this will be returned to HA as a device_class
        # https://developers.home-assistant.io/docs/core/entity/binary-sensor
        self._sensor_type: Optional[Text] = None
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()
        self._device_class: Optional[Text] = None
        self.measurement: Optional[Text] = None
        self.enabled_by_default = False

    async def async_update(self, wake_if_asleep=False, force=False) -> None:
        """Update the vehicle data."""
        await super().async_update(wake_if_asleep=wake_if_asleep)
        self.refresh()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.__state = self.vin()

    def get_value(self) -> Optional[str]:
        """Return the state."""
        return self.__state

    @property
    def device_class(self) -> Optional[Text]:
        """Return the HA device class."""
        return self._device_class

    @classmethod
    def _dict_to_attr(
        cls,
        data: dict,
        exclude_dicts: Optional[List[str]] = None,
        prepend: Optional[str] = None,
    ) -> dict:
        """Convert Tesla returned dict into dict for attributes."""
        prepend = prepend or ""
        exclude_dicts = exclude_dicts or []

        attr: dict = {}
        for key, value in data.items():
            prepend_key = f"{key}" if prepend == "" else f"{prepend}_{key}"
            if isinstance(value, dict):
                if key in exclude_dicts or "*" in exclude_dicts:
                    continue
                attr.update(cls._dict_to_attr(value, exclude_dicts, prepend_key))
            else:

                attr[prepend_key] = value

        return attr


class ClimateStateDataSensor(VehicleDataSensor):
    """Home-Assistant Climate State sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the ClimateStateData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "climate state data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()

        self.attrs = self._dict_to_attr(self._controller.get_climate_params(self._id))


class ChargeStateDataSensor(VehicleDataSensor):
    """Home-Assistant Charge State sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the ChargeStateData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "charging state data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(self._controller.get_charging_params(self._id))


class VehicleStateDataSensor(VehicleDataSensor):
    """Home-Assistant Vehicle State sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the VehicleStateData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "vehicle state data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(
            self._controller.get_state_params(self._id),
            ["software_update", "speed_limit_mode"],
        )


class SoftwareDataSensor(VehicleDataSensor):
    """Home-Assistant Software sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the SoftwareData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "software data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(
            self._controller.get_state_params(self._id).get("software_update", {})
        )


class SpeedLimitDataSensor(VehicleDataSensor):
    """Home-Assistant Speed Limit sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the SpeedLimitData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "speed limit data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(
            self._controller.get_state_params(self._id).get("speed_limit_mode", {})
        )


class VehicleConfigDataSensor(VehicleDataSensor):
    """Home-Assistant Vehicle Config sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the VehicleConfigData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "vehicle config data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(self._controller.get_config_params(self._id))


class DriveStateDataSensor(VehicleDataSensor):
    """Home-Assistant Drive State sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the DriveStateData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "drive state data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(self._controller.get_drive_params(self._id))


class GuiSettingsDataSensor(VehicleDataSensor):
    """Home-Assistant GUI settings sensor class for a Tesla VehicleDevice."""

    def __init__(self, data: dict, controller) -> None:
        """Initialize the GuiSettingsData sensor.

        Args:
            data (Dict): The base state for a Tesla vehicle.
                https://tesla-api.timdorr.com/vehicle/state/data
            controller (Controller): The controller that controls updates to the Tesla API.

        """
        super().__init__(data, controller)

        self.type: Text = "gui settings data sensor"
        self.name: Text = self._name()
        self.uniq_name: Text = self._uniq_name()

    def refresh(self) -> None:
        """Refresh data.

        This assumes the controller has already been updated
        """
        super().refresh()
        self.attrs = self._dict_to_attr(self._controller.get_gui_params(self._id))
